unit uDispatchImpl;

interface

uses
	Windows, SysUtils, Classes, ActiveX, ComObj;

type
	POleVariantArray = ^TOleVariantArray;
	TOleVariantArray = array[0..65535] of OleVariant;

	TInvokeFunction = function(DispID: Integer; const IID: TGUID; LocaleID: Integer;
	  Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult of Object;

	TInvokeFunctionSmart = function( DispID: Integer; const Flags: Word; Params: POleVariantArray; Argc: Integer; var VarResult, ExcepInfo, ArgErr: OleVariant ): HResult of Object;

	TInvokeListItem = class( TCollectionItem )
	private
		FInvokeFunction: TInvokeFunction;
		FAcceptFlags: Word;
		FName: WideString;
	end;

	TInvokeList = class( TCollection )
	private
		function GetItem(Index: Integer): TInvokeListItem;
	public
		constructor Create();
		function Add( const FunctionName: WideString; const AcceptFlags: Word; const Proc: TInvokeFunction ): TInvokeListItem; reintroduce;
		function Insert( const Index: Integer; const FunctionName: WideString; const AcceptFlags: Word; const Proc: TInvokeFunction ): TInvokeListItem; reintroduce;
		property Items[Index: Integer]: TInvokeListItem read GetItem; default;
	end;

	TDispatchImpl = class(TInterfacedObject, IDispatch)
	private
		FInvokeList: TInvokeList;
	protected
		function GetTypeInfoCount(out Count: Integer): HResult; stdcall;
		function GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult; stdcall;
		function GetIDsOfNames(const IID: TGUID; Names: Pointer; NameCount, LocaleID: Integer; DispIDs: Pointer): HResult; stdcall;
		function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; virtual; stdcall;
		procedure AddFunction( const FunctionName: WideString; const AcceptFlags: Word; const Proc: TInvokeFunction );
		function Default(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; virtual;
	public
		constructor Create;
		destructor Destroy; override;
	end;

	TSmartDispatchImpl = class( TDispatchImpl )
	public
		constructor Create;
	protected
		function Default( DispID: Integer; const Flags: Word; Params: POleVariantArray; Argc: Integer; var VarResult, ExcepInfo, ArgErr: OleVariant ): HResult; reintroduce; virtual;
		function Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult; override; stdcall;
		procedure AddFunction( const FunctionName: WideString; const AcceptFlags: Word; const Proc: TInvokeFunctionSmart ); reintroduce;
	end;

implementation

{ TInvokeList }

function TInvokeList.Add( const FunctionName: WideString; const AcceptFlags: Word; const Proc: TInvokeFunction ): TInvokeListItem;
begin
	Result := TInvokeListItem( inherited Add() );
	Result.FName := UpperCase( FunctionName );
	Result.FInvokeFunction := Proc;
	Result.FAcceptFlags := AcceptFlags;
end;

constructor TInvokeList.Create;
begin
	inherited Create( TInvokeListItem );
end;

function TInvokeList.GetItem(Index: Integer): TInvokeListItem;
begin
	Result := TInvokeListItem( inherited GetItem( Index ) );
end;

function TInvokeList.Insert(const Index: Integer; const FunctionName: WideString; const AcceptFlags: Word; const Proc: TInvokeFunction ): TInvokeListItem;
begin
	Result := TInvokeListItem( inherited Insert( Index ) );
	Result.FName := UpperCase( FunctionName );
	Result.FInvokeFunction := Proc;
	Result.FAcceptFlags := AcceptFlags;
end;

{ TDispatchImpl }

procedure TDispatchImpl.AddFunction( const FunctionName: WideString; const AcceptFlags: Word; const Proc: TInvokeFunction);
begin
	FInvokeList.Add( FunctionName, AcceptFlags, Proc );
end;

constructor TDispatchImpl.Create;
begin
	FInvokeList := TInvokeList.Create();
	FInvokeList.Add( '', DISPATCH_PROPERTYGET, Default );
end;

function TDispatchImpl.Default(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult;
begin
	POleVariant(VarResult)^ := IDispatch( Self );
	Result := S_OK;
end;

destructor TDispatchImpl.Destroy;
begin
	FInvokeList.Free;
	inherited;
end;

function TDispatchImpl.GetIDsOfNames(const IID: TGUID; Names: Pointer; NameCount, LocaleID: Integer; DispIDs: Pointer): HResult;
type
	TDispIDsArray = array[0..0] of TDISPID;
	PDispIDsArray = ^TDispIDsArray;
var
	IDs: PDispIDsArray absolute DispIDs;
	i: integer;
	Name: WideString;
begin
	if NameCount > FInvokeList.Count then
		Result := DISP_E_UNKNOWNNAME
	else if NameCount < 1 then
		Result := E_INVALIDARG
	else
		Result := S_OK;

	for i := 0 to NameCount - 1 do
		IDs[i] := DISPID_UNKNOWN;

	if NameCount = 1 then begin
		Name := UpperCase( PWideChar(Names^) );
		for i := 0 to FInvokeList.Count-1 do
			if FInvokeList[i].FName = Name then begin
				IDs[0] := i;
				Exit;
			end;
	   Result := DISP_E_UNKNOWNNAME;
	end;
end;

function TDispatchImpl.GetTypeInfo(Index, LocaleID: Integer; out TypeInfo): HResult;
begin
	Pointer( TypeInfo ) := nil;
	Result := E_NOTIMPL;
end;

function TDispatchImpl.GetTypeInfoCount( out Count: Integer): HResult;
begin
	Count := 0;
	Result := S_OK;
end;

function TDispatchImpl.Invoke(DispID: Integer; const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult, ExcepInfo, ArgErr: Pointer): HResult;
begin
	Result := DISP_E_MEMBERNOTFOUND;
	if ( FInvokeList.Count <= DispID ) then Exit;
	if ( FInvokeList[DispID].FAcceptFlags and Flags ) = 0 then Exit;
	Result := FInvokeList[DispID].FInvokeFunction( DispID, IID, LocaleID, Flags, Params, VarResult, ExcepInfo, ArgErr );
end;

{ TSmartDispatchImpl }

procedure TSmartDispatchImpl.AddFunction( const FunctionName: WideString; const AcceptFlags: Word; const Proc: TInvokeFunctionSmart );
begin
	//DISPATCH_METHOD         = $1;
	//DISPATCH_PROPERTYGET    = $2;
	//DISPATCH_PROPERTYPUT    = $4;
	//DISPATCH_PROPERTYPUTREF = $8;
	FInvokeList.Add( FunctionName, AcceptFlags, TInvokeFunction( Proc ) );
end;

constructor TSmartDispatchImpl.Create;
begin
	inherited Create();
	FInvokeList.Clear();
	AddFunction( '', DISPATCH_PROPERTYGET, Default );
end;

function TSmartDispatchImpl.Default(DispID: Integer; const Flags: Word; Params: POleVariantArray; Argc: Integer; var VarResult, ExcepInfo, ArgErr: OleVariant): HResult;
begin
	VarResult := IDispatch( Self );
	Result := S_OK;
end;

function TSmartDispatchImpl.Invoke(DispID: Integer;
  const IID: TGUID; LocaleID: Integer; Flags: Word; var Params; VarResult,
  ExcepInfo, ArgErr: Pointer): HResult;
var
	f: TInvokeFunctionSmart;
	Parms: PDispParams;
	a2: POleVariantArray;
	a: array of OleVariant;
	i: Integer;
	t: OleVariant;
	r: OleVariant;
begin
	Result := DISP_E_MEMBERNOTFOUND;
	if ( FInvokeList.Count <= DispID ) then Exit;
	f := TInvokeFunctionSmart( FInvokeList[DispID].FInvokeFunction );
	Parms := @Params;
	a2 := POleVariantArray(Parms.rgvarg);
	SetLength( a, Parms.cArgs );
	for i := 0 to Parms.cArgs - 1 do begin
		a[i] := a2[ Parms.cArgs-1-i ];
	end;
	if not Assigned( VarResult ) then VarResult := @r;
	Result := f( DispID, Flags, @(a[0]), Parms.cArgs, POleVariant(VarResult)^, POleVariant(ExcepInfo)^, t );
end;

end.

