unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, PNGImage { TPNGObject }, uProcessList,
  StdCtrls, uMSNMCheck, uActionBase, uResizeSurface, Menus, uMessageWindow,
  uSisterHEART, uScriptEngine, ExtCtrls;

type
  TIMOMain = class(TForm)
    Popup: TPopupMenu;
    N1: TMenuItem;
    N2: TMenuItem;
    Exit1: TMenuItem;
    SayTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormPaint(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure N1Click(Sender: TObject);
    procedure SayTimerTimer(Sender: TObject);
  private
  	procedure RemakeSurface();
    { Private 錾 }
  public
  	MessageWindow: TMessageWindow;
  	ZoomRate: Double;
  	Tex: TBitmap;
    procedure SayCommand( const Message: WideString );
    { Public 錾 }
  end;

var
  IMOMain: TIMOMain;

implementation

uses
	uImageProcessing;

{$R *.dfm}

procedure SetLayered( const hWnd: THandle; const Enable: Boolean );
begin
	if Enable then
		SetWindowLong( hWnd, GWL_EXSTYLE, GetWindowLong( hWnd, GWL_EXSTYLE ) or WS_EX_LAYERED )
	else
		SetWindowLong( hWnd, GWL_EXSTYLE, GetWindowLong( hWnd, GWL_EXSTYLE ) and not(WS_EX_LAYERED) );
end;

function SetLayeredWindowBuffer( hWnd: THandle; BMP: TBitmap; SrcPos, DestPos: TPoint; ASize: TSize; ColorKey: DWORD=0; AlphaLevel: Byte = 255 ): Boolean; overload;
var
	bf: TBlendFunction;
	h, h2, hbmp: THandle;
begin
	with bf do begin
		BlendOp := AC_SRC_OVER;
		AlphaFormat := AC_SRC_ALPHA;
		BlendFlags := 0;
		SourceConstantAlpha := AlphaLevel;
	end;

	h := GetDC( 0 );
	try
		h2 := CreateCompatibleDC( h );
		try
			hbmp := SelectObject( h2, BMP.Handle );
			try
				Result := UpdateLayeredWindow( hWnd, h, @DestPos, @ASize, h2, @SrcPos, 0, @bf, ULW_ALPHA );
			finally
				SelectObject( h2, hbmp );
			end;
		finally
			DeleteDC( h2 );
		end;
	finally
		ReleaseDC( 0, h );
	end;
end;

function SetLayeredWindowBuffer( hWnd: THandle; BMP: TBitmap; AlphaLevel: SmallInt = 255 ): Boolean; overload;
	function Point( x, y: Integer ): TPoint;
	begin
		Result.X := x;
		Result.Y := y;
	end;
	function Size( x, y: Integer ): TSize;
	begin
		Result.cx := x;
		Result.cy := y;
	end;
var
	r: TRect;
begin
	GetWindowRect( hWnd, r );
	Result := SetLayeredWindowBuffer( hWnd, BMP, Point( 0, 0 ), r.TopLeft, Size(  r.Right-r.Left, r.Bottom-r.Top ), 0, Saturate( AlphaLevel ) );
end;

procedure TIMOMain.FormCreate(Sender: TObject);
begin
	Randomize();

    ZoomRate := 0.75;
	BorderStyle := bsNone;
	SetLayered( Handle, True );
	RemakeSurface();
    Left := Screen.WorkAreaLeft + Screen.WorkAreaWidth - Width - 32;
    Top := Screen.WorkAreaTop + Screen.WorkAreaHeight - Height;

    MessageWindow := TMessageWindow.Create( Self );
	MessageWindow.ResetPos();

    SisterHEART.SayCommand := SayCommand;

    ActionList.Add( TASAction.Create() );

	SayTimer.Enabled := True;
end;

procedure TIMOMain.FormDestroy(Sender: TObject);
begin
 	SayTimer.Enabled := False;

	Tex.Free;
    MessageWindow.Free;
end;

procedure TIMOMain.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
	ReleaseCapture();
	Perform( WM_SYSCOMMAND, SC_MOVE or 2, 0 );
end;

procedure TIMOMain.FormPaint(Sender: TObject);
begin
	Canvas.Draw( 0, 0, Tex );
end;

procedure TIMOMain.Exit1Click(Sender: TObject);
begin
	Close();
end;

procedure TIMOMain.N1Click(Sender: TObject);
begin
	SayTimer.Enabled := False;

	with TResizeSurface.Create( Self ) do
    try
    	TrackBar1.Position := Round( ZoomRate * 100 );
    	ShowModal();
        ZoomRate := TrackBar1.Position / 100;
    finally
    	Free;
    end;
	RemakeSurface();

	SayTimer.Enabled := True;
end;

procedure TIMOMain.RemakeSurface;
var
	v: Boolean;
	png: TPNGObject;
    tmp, tmpTex: TBitmap;
    Pos: TPoint;
begin
	v := Visible;
	Pos := Point( Left + Width div 2, Top + Height div 2 );
	png := TPNGObject.Create();
    try
		tmp := TBitmap.Create();
        try
	    	png.LoadFromFile( 'imo.png' );
        	CompleteAssign( tmp, png );
	        tmpTex := Resize32BitBitmap( tmp, ZoomRate );
        finally
        	tmp.Free;
        end;
    finally
    	png.Free;
    end;
    MakeAlphaCelcedBMP( tmpTex, 0, 0, tmpTex.Width, tmpTex.Height, tmpTex, 0, 0 );
    Visible := False;
    ClientWidth := tmpTex.Width;
    ClientHeight := tmpTex.Height;
    Left := Pos.X - Width div 2;
    Top := Pos.Y - Height div 2;
    SetLayeredWindowBuffer( Handle, tmpTex, 255 );
    Visible := v;

	if Assigned( Tex ) then FreeAndNil( Tex );
	Tex := tmpTex;
end;

procedure TIMOMain.SayCommand(const Message: WideString);
begin
	MessageWindow.Talk( Message );
	with MessageWindow, Application do
		while Visible and( not Terminated ) do ProcessMessages();
end;

procedure TIMOMain.SayTimerTimer(Sender: TObject);
var
	i: ISisterAction;
begin
	SayTimer.Enabled := False;
	ActionList[ Random( ActionList.Count ) ].QueryInterface( ISisterAction, i );
    if Assigned( i ) then i.Execute();
	SayTimer.Enabled := True;
end;

end.
