unit uMessageWindow;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls;

type
  TMessageWindow = class(TForm)
    HideTimer: TTimer;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormClick(Sender: TObject);
    procedure HideTimerTimer(Sender: TObject);
  private
  	FParent: TForm;
    FParentOldWindowProc: TWndMethod;
    procedure ParentNewWindowProc( var Message: TMessage );
    { Private 錾 }
  public
  	constructor Create( const Parent: TForm ); reintroduce;
	procedure SetNewSize();
    procedure ResetPos();
    procedure Hide(); reintroduce;
    procedure Talk( const Message: string );
  end;

var
  MessageWindow: TMessageWindow;

implementation

{$R *.dfm}

procedure TMessageWindow.FormCreate(Sender: TObject);
begin
	SetNewSize();
end;

procedure TMessageWindow.SetNewSize;
begin
	SetWindowRgn( Handle, CreateRoundRectRgn( 0, 0, Width+1, Height+1, 11, 11 ), True );
end;

procedure TMessageWindow.FormPaint(Sender: TObject);
var
	r: HRGN;
begin
	with Canvas do begin
		r := CreateRectRgn( 0, 0, 0, 0 );
		Brush.Color := clInfoText;
		if GetWindowRgn( Self.Handle, r ) <> ERROR then
			FrameRgn( Handle, r, Canvas.Brush.handle, 1, 1 )
		else
			FrameRect( Rect( 0, 0, Width, Height ) );
		DeleteObject( r );
	end;
end;

procedure TMessageWindow.Hide;
begin
	inherited Hide();
	HideTimer.Enabled := False;
end;

constructor TMessageWindow.Create(const Parent: TForm);
begin
	inherited Create( Parent );

    FParent := Parent;
    FParentOldWindowProc := FParent.WindowProc;
    FParent.WindowProc := ParentNewWindowProc;
end;

procedure TMessageWindow.ParentNewWindowProc(var Message: TMessage);
var
	pr: PRect;
    d: Integer;
begin
	FParentOldWindowProc( Message );
    case Message.Msg of
    	WM_MOVING: begin
        	pr := Pointer( Message.lParam );
            d := Screen.WorkAreaTop + Screen.WorkAreaHeight - ( pr.Bottom - pr.Top );
            OffsetRect( pr^, 0, d - pr.Top );
            Message.Result := 1;
        end;
    	WM_MOVE: begin
        	FParent.Top := Screen.WorkAreaTop + Screen.WorkAreaHeight - FParent.Height;
            Message.Result := 0;
        	ResetPos();
        end;
    end;
end;

procedure TMessageWindow.ResetPos;
begin
	if Screen.WorkAreaLeft + Screen.WorkAreaWidth > FParent.Left + FParent.Width + 16 + Width then
		Left := FParent.Left + FParent.Width + 16
	else
		Left := FParent.Left - 16 - Width;
	Top := FParent.Top;
end;

procedure TMessageWindow.FormClick(Sender: TObject);
begin
	Hide();
end;

procedure TMessageWindow.HideTimerTimer(Sender: TObject);
begin
	Hide();
end;

procedure TMessageWindow.Talk(const Message: string);
begin
	Label1.Caption := StringReplace( Message, '\n', #13#10, [rfReplaceAll] );
    ResetPos();
    Visible := True;
    HideTimer.Interval := Length( Message )*50 + 3000;
	HideTimer.Enabled := True;
end;

end.
